package com.yaoyicloud.render;

import java.io.IOException;

import java.util.Map;


import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yaoyicloud.config.FilerepoProperties;
import com.yaoyicloud.message.FxyProtos;
import lombok.extern.slf4j.Slf4j;

import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;

import com.google.protobuf.util.JsonFormat;


/**
 * FinancialInfo渲染器
 *
 */
@Slf4j
public final class FinancialInfoRender extends AbstractRender {
    private final FilerepoProperties filerepoProperties;
    public FinancialInfoRender(String cwd, FilerepoProperties filerepoProperties) {
        super(cwd);
        this.filerepoProperties = filerepoProperties;
    }
    @Override
    protected String getBasicPath() throws IOException {
        return filerepoProperties.getBasePath();
    }

    @Override
    protected String getReportImagePath() {
        return filerepoProperties.getReportImagePath();
    }

    /**
     * Docx 渲染
     *
     * @param info 数据
     * @param templateFileContent 模板内容
     * @return 本地文件目录
     * @throws IOException
     */

    public String renderDocx(String info, Map<String, Object> addtionalMap, byte[] templateFileContent, String relationId) throws IOException {
        log.info("开始渲染财务模块，relationId: {}", relationId);

        // 配置POI-TL渲染器
        ConfigureBuilder builder = Configure.builder();
        builder.useSpringEL();

        builder.bind("indicators", new LoopRowIncludeStatisticsTableRenderPolicy("values"));
        builder.bind("financialDataSeq", new LoopColumnStaticTableRenderPolicy("[", "]", false, true, 2));
        builder.bind("financialCheckDetails", new LoopRowCutAndMergeFirstColTableRenderPolicy());
        FxyProtos.FinancialInfo.Builder basicInfoBuilder = FxyProtos.FinancialInfo.newBuilder();
        JsonFormat.parser().merge(info, basicInfoBuilder);

        FxyProtos.FinancialInfo defaultInstance = FxyProtos.FinancialInfo.getDefaultInstance();
        FxyProtos.FinancialInfo mergedProto = defaultInstance.toBuilder()
                .mergeFrom(basicInfoBuilder.build())
                .build();

        String completeJson = JsonFormat.printer()
                .includingDefaultValueFields()
                .print(mergedProto);
        ObjectMapper objectMapper = new ObjectMapper();
        Map<String, Object> data = objectMapper.readValue(completeJson, new TypeReference<Map<String, Object>>() {});
        data.replaceAll((k, v) -> v.equals("") ? "-" : v);

        if (addtionalMap != null) {
            data.putAll(addtionalMap);
        }
        fillBasicDefaultValues(data);

        try {
            // 渲染文档
            String resultPath = this.renderDocx(data, templateFileContent, builder, relationId, "financialInfo");
            log.info("渲染财务模块成功，文件路径: {}", resultPath);
            return resultPath;
        } catch (Exception e) {
            log.error("渲染财务模块失败，relationId: {}", relationId, e);
            throw new IOException("文档渲染失败", e);
        }
    }
    /**
     * 填充默认值，确保所有必要字段都存在
     */
    private void fillBasicDefaultValues(Map<String, Object> data) {
        Map<String, Object> basicInfoSummary = (Map<String, Object>) data.get("financialSummary");
        basicInfoSummary.replaceAll((k, v) -> v.equals("") ? "-" : v);
    }



}
